// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 17 1.2.1.3 source/core/collect/ibtree.inl, collection, ioc.v400, trackPseudoLevel 
#ifndef _IBTREE_INL_
#define _IBTREE_INL_

// -----------
// ITreeCursor
// -----------

// public members

inline void
ITreeCursor::
copy (ITreeCursor const& cursor)
{ ImplOf (*this).Copy (*cursor.ivImpl);
}

inline void
ITreeCursor::
invalidate ()
{ ImplOf (*this).Invalidate ();
}

inline IBool
ITreeCursor::
isValid () const
{ return ImplOf (*this).IsValid ();
}

inline IBool
ITreeCursor::
setToChild (IPosition position)
{ return ImplOf (*this).SetToChild (position);
}

inline IBool
ITreeCursor::
setToFirstExistingChild ()
{ return ImplOf (*this).SetToFirstExistingChild ();
}

inline IBool
ITreeCursor::
setToLastExistingChild ()
{ return ImplOf (*this).SetToLastExistingChild ();
}

inline IBool
ITreeCursor::
setToNextExistingChild ()
{ return ImplOf (*this).SetToNextExistingChild ();
}

inline IBool
ITreeCursor::
setToParent ()
{ return ImplOf (*this).SetToParent ();
}

inline IBool
ITreeCursor::
setToPreviousExistingChild ()
{ return ImplOf (*this).SetToPreviousExistingChild ();
}

inline IBool
ITreeCursor::
setToRoot ()
{ return ImplOf (*this).SetToRoot ();
}

inline void
ITreeCursor::
operator= (ITreeCursor const& cursor)
{ ImplOf (*this).Copy (ImplOf (cursor));
}

inline IBool
ITreeCursor::
operator== (ITreeCursor const& cursor) const
{ return ImplOf (*this).operator== (*cursor.ivImpl);
}

inline IBool
ITreeCursor::
operator!= (ITreeCursor const& cursor) const
{ return ImplOf (*this).operator!= (*cursor.ivImpl);
}

// protected members

inline
ITreeCursor::
ITreeCursor (Implementation* impl)
: ivImpl (impl)
{
}

inline
ITreeCursor::
ITreeCursor (ITreeCursor const& cursor)
: ivImpl (ImplOf (cursor).Clone ())
{
}

inline ITreeCursor::Implementation&
ITreeCursor::
ImplOf (ITreeCursor const& cursor)
{ return *(Implementation*)cursor.ivImpl;
}

// ----------
// IATreeBase
// ----------

// protected members

inline
IATreeBase::
IATreeBase ()
: IMetaTree ()
{
}

inline
IATreeBase::
IATreeBase (IATreeBase const& tree)
: IMetaTree (tree)
{
}

inline
IATreeBase::
~IATreeBase ()
{
}

inline ITreeCursorImpl&
IATreeBase::
CrsrImplOf (ITreeCursor& cursor)
{ return *cursor.ivImpl;
}

inline ITreeCursorImpl const&
IATreeBase::
CrsrImplOf (ITreeCursor const& cursor)
{ return *cursor.ivImpl;
}

#endif // _IBTREE_INL_
