// ----------------------------------------------------------------------------
// FILE NAME: idate.inl
//
// DESCRIPTION:
//   This file contains the definition of the inline functions for the
//   classes declared in idate.hpp.
//
//
// COPYRIGHT:
//   IBM Open Class Library
//   Licensed Materials - Property of IBM
//                                                                            
// Revision: 96  1.8.1.4  source/core/base/idate.inl, appsupport, ioc.v400, trackPseudoLevel  
// ----------------------------------------------------------------------------

#ifndef _IDATE_INL_
#define _IDATE_INL_

extern "C"
{
    #include <time.h>
}

inline IDate::IDate(Month aMonth, int aDay, int aYear)
{
    this->initialize(aMonth, aDay, aYear);
}

inline IDate::IDate(int aDay, Month aMonth, int aYear)
{
    this->initialize(aMonth, aDay, aYear);
}

inline IDate::IDate(const IDate& aDate) :

    julian(aDate.julian)
{
}

inline IDate::IDate(unsigned long julianDayNumber) :

    julian(julianDayNumber)
{
}

inline IDate::DayOfWeek IDate::dayOfWeek() const
{
    return DayOfWeek(this->julian % 7);
}

inline unsigned long IDate::julianDate() const
{
    return this->julian;
}

inline IBool IDate::operator==(const IDate& aDate) const
{
    return this->julian == aDate.julian;
}

inline IBool IDate::operator!=(const IDate& aDate) const
{
    return this->julian != aDate.julian;
}

inline IBool IDate::operator<(const IDate& aDate) const
{
    return this->julian <  aDate.julian;
}

inline IBool IDate::operator<=(const IDate& aDate) const
{
    return this->julian <= aDate.julian;
}

inline IBool IDate::operator>(const IDate& aDate) const
{
    return this->julian >  aDate.julian;
}

inline IBool IDate::operator>=(const IDate& aDate) const
{
    return this->julian >= aDate.julian;
}

inline IDate IDate::operator+(int numDays) const
{
    return IDate( this->julian + numDays );
}

inline IDate IDate::operator-(int numDays) const
{
    return IDate(this->julian - numDays);
}

inline IDate& IDate::operator+=(int numDays)
{
    this->julian += numDays;
    return *this;
}

inline IDate& IDate::operator-=(int numDays)
{
    this->julian -= numDays;
    return *this;
}

inline long IDate::operator-(const IDate& aDate) const
{
    return this->julian - aDate.julian;
}

inline IBool IDate::isValid(int aDay, Month aMonth, int aYear)
{
    return IDate::isValid(aMonth, aDay, aYear);
}

inline IBool IDate::isValid(int aYear, int  aDay)
{
    return(aYear != 0
           &&
           aDay <= IDate::daysInYear(aYear));
}

inline void IDate::operator>>=(IDataStream &toWhere)
{
    writeToStream(toWhere);
}

inline void IDate::operator<<=(IDataStream &fromWhere)
{
    readFromStream(fromWhere);
}

inline ostream& operator<<(ostream& aStream, const IDate& aDate)
{
    return aStream << aDate.asString();
}

#endif // _IDATE_INL_
