// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 00 1.3.1.3 source/core/collect/iiatree.inl, collection, ioc.v400, trackPseudoLevel 
#ifndef _IIATREE_INL_
#define _IIATREE_INL_


#include <iiexc.h>
// -----------------
// ITreeResourceImpl
// -----------------

inline
ITreeResourceImpl::
~ITreeResourceImpl ()
{ delete ivResource;
}


// ---------------
// ITreeCursorImpl
// ---------------

// public members

inline
ITreeCursorImpl::
ITreeCursorImpl (IATreeImpl const& tree)
: ivTree (&tree)
{
}

inline
ITreeCursorImpl::
~ITreeCursorImpl ()
{
}

inline void const*
ITreeCursorImpl::
Element () const
{ return ivTree->ElementAt (*this);
}

inline IBool
ITreeCursorImpl::
IsFor (IATreeImpl const& tree) const
{ return (ivTree == &tree);
}

inline IBool
ITreeCursorImpl::
SetToChild (IPosition position)
{ return ivTree->SetToChild (position, *this);
}

inline IBool
ITreeCursorImpl::
SetToFirstExistingChild ()
{ return ivTree->SetToFirstExistingChild (*this);
}

inline IBool
ITreeCursorImpl::
SetToLastExistingChild ()
{ return ivTree->SetToLastExistingChild (*this);
}

inline IBool
ITreeCursorImpl::
SetToNextExistingChild ()
{ return ivTree->SetToNextExistingChild (*this);
}

inline IBool
ITreeCursorImpl::
SetToParent ()
{ return ivTree->SetToParent (*this);
}

inline IBool
ITreeCursorImpl::
SetToPreviousExistingChild ()
{ return ivTree->SetToPreviousExistingChild (*this);
}

inline IBool
ITreeCursorImpl::
SetToRoot ()
{ return ivTree->SetToRoot (*this);
}

inline void
ITreeCursorImpl::
operator= (ITreeCursorImpl const& cursor)
{ ivTree = cursor.ivTree;
}

inline IBool
ITreeCursorImpl::
operator== (ITreeCursorImpl const& cursor) const
{ return (ivTree == cursor.ivTree);
}

inline IBool
ITreeCursorImpl::
operator!= (ITreeCursorImpl const& cursor) const
{ return ! operator== (cursor);
}

// protected members

inline void*
ITreeCursorImpl::
CheckPointer (void* ptr)
{ return IATreeImpl::CheckPointer (ptr);
}

// ----------
// IATreeImpl
// ----------

// public members

inline
IATreeImpl::
~IATreeImpl ()
{
}

// protected members

inline
IATreeImpl::
IATreeImpl (Ops& ops)
: ivOps (&ops), ivReferenceCount (0)

  ,ivResource (0)

#if ! defined (__INO_STREAMING__)
  ,ivStreamer (0)
#endif
{
}

inline
IATreeImpl::
IATreeImpl (IATreeImpl const&)
{ // should never be executed
}

inline IATreeImpl::Ops&
IATreeImpl::
OpsOf (IATreeImpl const& tree)
{ return *tree.ivOps;
}

#endif // _IIATREE_INL_
