// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 12 1.4.1.3 source/core/collect/iiseqtab.inl, collection, ioc.v400, trackPseudoLevel 
#ifndef _IISEQTAB_INL_
#define _IISEQTAB_INL_

// ------------------------
// ISequenceAsTableNodeImpl
// ------------------------

// public member

template <class Element>
inline
ISequenceAsTableNodeImpl <Element>::
ISequenceAsTableNodeImpl (Element const& element)
: ivElement (element)
{
}

#if ! defined(__INO_STREAMING__)
template <class Element>
inline void
ISequenceAsTableNodeImpl<Element>::
StreamIn (IACollectionStreamer *streamer, IDataStream& fromWhere)
{ streamer->streamIn(fromWhere, &ivElement);
}
#endif

template <class Element>
inline
ISequenceAsTableNodeImpl <Element>::
~ISequenceAsTableNodeImpl ()
{
}

// --------------------------
// ISequenceAsTableCursorImpl
// --------------------------

// public members

inline void
ISequenceAsTableCursorImpl::
Invalidate ()
{ ivIndex = 0;
}

inline IBool
ISequenceAsTableCursorImpl::
IsInBetween () const
{ return false;
}

inline IBool
ISequenceAsTableCursorImpl::
IsValid () const
{ return (ivIndex != 0);
}

inline void
ISequenceAsTableCursorImpl::
operator= (ISequenceAsTableCursorImpl const& cursor)
{ Inherited::operator= (cursor);
  ivIndex = cursor.ivIndex;
}

inline IBool
ISequenceAsTableCursorImpl::
operator== (ICursorImpl const& cursor) const
{ return (Inherited::operator== (cursor) &&
          ivIndex == ((Self const&)cursor).ivIndex);
}

// --------------------
// IASequenceAsTableOps
// --------------------

// public members

inline
IASequenceAsTableOps::
~IASequenceAsTableOps ()
{
}

// protected members

inline
IASequenceAsTableOps::
IASequenceAsTableOps (IBool containsDTSObjects)
: ivContainsDTSObjects (containsDTSObjects)
{
}

inline ITableImpl&
IASequenceAsTableOps::
TableOf (ISequenceAsTableImpl const& collection)
{ return ((ISequenceAsTableImpl&)collection).ivTable;
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForAssign (ISequenceAsTableImpl const& collection,
                           INumber target, INumber source) const
{ collection.UpdateMngCursorsForAssign (target, source);
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForCopyFrom (ISequenceAsTableImpl const& collection,
                             INumber fromIndex, INumber toIndex) const
{ collection.UpdateMngCursorsForCopyFrom (fromIndex, toIndex);
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForPredRemove (ISequenceAsTableImpl const& collection,
                               INumber removed, INumber previous) const
{ collection.UpdateMngCursorsForPredRemove (removed, previous);
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForRemoveOf (ISequenceAsTableImpl const& collection,
                             INumber index) const
{ collection.UpdateMngCursorsForRemoveOf (index);
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForShiftLeftOf (ISequenceAsTableImpl const& collection,
                                INumber index) const
{ collection.UpdateMngCursorsForShiftLeftOf (index);
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForShiftRightOf (ISequenceAsTableImpl const& collection,
                                 INumber index) const
{ collection.UpdateMngCursorsForShiftRightOf (index);
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForSwap (ISequenceAsTableImpl const& collection,
                         INumber index1, INumber index2) const
{ collection.UpdateMngCursorsForSwap (index1, index2);
}

#endif // _IISEQTAB_INL_
